@@ -1,78 +1,231 @@
+#include <float.h> /* DBL_EPSILON */
 #include "dgeMatrix.h"
-// -> Mutils.h  etc
 
-SEXP dMatrix_validate(SEXP obj)
+/* FIXME: also allow an interface to LAPACK's 'dgesvx' which performs 
+          LU factorization then optionally does "equilibration" (row and
+	  column scaling) ... maybe also allow an interface to 'dgeequ' ... 
+*/
+
+SEXP dgeMatrix_trf_(SEXP obj, int warn)
 {
-    SEXP x = GET_SLOT(obj, Matrix_xSym),
-	Dim = GET_SLOT(obj, Matrix_DimSym);
-    if (!isReal(x))
-	return mkString(_("x slot must be numeric \"double\""));
     SEXP val;
-    if (isString(val = dim_validate(Dim, "Matrix")))
+    PROTECT_INDEX pidA;
+    PROTECT_WITH_INDEX(val = get_factor(obj, "LU"), &pidA);
+    if (!isNull(val)) {
+	UNPROTECT(1);
 	return val;
-    return ScalarLogical(1);
-}
-
-SEXP dgeMatrix_validate(SEXP obj)
-{
-    SEXP val;
-    if (isString(val = dim_validate(GET_SLOT(obj, Matrix_DimSym), "dgeMatrix")))
-	return(val);
-    if (isString(val = dense_nonpacked_validate(obj)))
-	return(val);
-    SEXP fact = GET_SLOT(obj, Matrix_factorSym);
-    if (length(fact) > 0 && getAttrib(fact, R_NamesSymbol) == R_NilValue)
-	return mkString(_("factors slot must be named list"));
-    return ScalarLogical(1);
-}
-
-static
-double get_norm(SEXP obj, const char *typstr)
-{
-    if(any_NA_in_x(obj))
-	return NA_REAL;
-    else {
-	char typnm[] = {'\0', '\0'};
-	int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
-	double *work = (double *) NULL;
-
-	typnm[0] = La_norm_type(typstr);
-	if (*typnm == 'I') {
-	    work = (double *) R_alloc(dims[0], sizeof(double));
-	}
-	return F77_CALL(dlange)(typstr, dims, dims+1,
-				REAL(GET_SLOT(obj, Matrix_xSym)),
-				dims, work FCONE);
     }
+    REPROTECT(val = NEW_OBJECT_OF_CLASS("denseLU"), pidA);
+    
+    SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym)),
+	dimnames = PROTECT(GET_SLOT(obj, Matrix_DimNamesSym));
+    int *pdim = INTEGER(dim), r = (pdim[0] < pdim[1]) ? pdim[0] : pdim[1];
+    SET_SLOT(val, Matrix_DimSym, dim);
+    SET_SLOT(val, Matrix_DimNamesSym, dimnames);
+
+    if (r > 0) {
+	PROTECT_INDEX pidB;
+	SEXP perm = PROTECT(allocVector(INTSXP, r)), x;
+	PROTECT_WITH_INDEX(x = GET_SLOT(obj, Matrix_xSym), &pidB);
+	REPROTECT(x = duplicate(x), pidB);
+	int *pperm = INTEGER(perm), info;
+	double *px = REAL(x);
+	
+	F77_CALL(dgetrf)(pdim, pdim + 1, px, pdim, pperm, &info);
+	
+	if (info < 0)
+	    error(_("LAPACK '%s' gave error code %d"),
+		  "dgetrf", info);
+	else if (info > 0 && warn > 0) {
+	    /* MJ: 'dgetrf' does not distinguish between singular, */
+	    /*     finite matrices and matrices containing NaN ... */
+	    /*     hence this message can mislead                  */
+	    if (warn > 1)
+		error  (_("LAPACK '%s': matrix is exactly singular, "
+			  "U[i,i]=0, i=%d"),
+			"dgetrf", info);
+	    else 
+		warning(_("LAPACK '%s': matrix is exactly singular, "
+			  "U[i,i]=0, i=%d"),
+			"dgetrf", info);
+	}
+	
+	SET_SLOT(val, Matrix_permSym, perm);
+	SET_SLOT(val, Matrix_xSym, x);
+	UNPROTECT(2);
+    }
+    
+    set_factor(obj, "LU", val);
+    UNPROTECT(3);
+    return val;
+}
+
+SEXP dgeMatrix_trf(SEXP obj, SEXP warn)
+{
+    return dgeMatrix_trf_(obj, asInteger(warn));
+}
+
+double get_norm_dge(SEXP obj, const char *typstr)
+{
+    SEXP x = PROTECT(GET_SLOT(obj, Matrix_xSym));
+    R_xlen_t i, nx = XLENGTH(x);
+    double *px = REAL(x);
+    for (i = 0; i < nx; ++i) {
+	if (ISNAN(px[i])) {
+	    UNPROTECT(1);
+	    return NA_REAL;
+	}
+    }
+    SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
+    int *pdim = INTEGER(dim);
+    double norm, *work = NULL;
+
+    if (typstr[0] == 'I')
+	work = (double *) R_alloc((size_t) pdim[0], sizeof(double));
+    norm = F77_CALL(dlange)(typstr, pdim, pdim + 1, px, pdim,
+			    work FCONE);
+    
+    UNPROTECT(2);
+    return norm;
 }
 
 SEXP dgeMatrix_norm(SEXP obj, SEXP type)
 {
-    return ScalarReal(get_norm(obj, CHAR(asChar(type))));
+    char typstr[] = {'\0', '\0'};
+    PROTECT(type = asChar(type));
+    typstr[0] = La_norm_type(CHAR(type));
+    double norm = get_norm_dge(obj, typstr);
+    UNPROTECT(1);
+    return ScalarReal(norm);
 }
 
 SEXP dgeMatrix_rcond(SEXP obj, SEXP type)
 {
-    SEXP LU = PROTECT(dgeMatrix_LU_(obj, FALSE));/* <- not warning about singularity */
-    char typnm[] = {'\0', '\0'};
-    int *dims = INTEGER(GET_SLOT(LU, Matrix_DimSym)), info;
-    double anorm, rcond;
+    SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
+    int *pdim = INTEGER(dim);
+    if (pdim[0] != pdim[1] || pdim[0] < 1)
+	error(_("'rcond' requires a square, nonempty matrix"));
 
-    if (dims[0] != dims[1] || dims[0] < 1) {
-	UNPROTECT(1);
-	error(_("rcond requires a square, non-empty matrix"));
-    }
-    typnm[0] = La_rcond_type(CHAR(asChar(type)));
-    anorm = get_norm(obj, typnm);
-    F77_CALL(dgecon)(typnm,
-		     dims, REAL(GET_SLOT(LU, Matrix_xSym)),
-		     dims, &anorm, &rcond,
-		     (double *) R_alloc(4*dims[0], sizeof(double)),
-		     (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
-    UNPROTECT(1);
+    char typstr[] = {'\0', '\0'};
+    PROTECT(type = asChar(type));
+    typstr[0] = La_rcond_type(CHAR(type));
+
+    SEXP trf = PROTECT(dgeMatrix_trf_(obj, 0)),
+	x = PROTECT(GET_SLOT(trf, Matrix_xSym));
+    int info;
+    double *px = REAL(x), norm = get_norm_dge(obj, typstr), rcond;
+    
+    F77_CALL(dgecon)(typstr, pdim, px, pdim, &norm, &rcond,
+		     (double *) R_alloc((size_t) 4 * pdim[0], sizeof(double)),
+		     (int *) R_alloc((size_t) pdim[0], sizeof(int)),
+		     &info FCONE);
+
+    UNPROTECT(4);
     return ScalarReal(rcond);
 }
 
+SEXP dgeMatrix_determinant(SEXP obj, SEXP logarithm)
+{
+    SEXP dim = PROTECT(GET_SLOT(obj, Matrix_DimSym));
+    int *pdim = INTEGER(dim), n = pdim[0];
+    if (pdim[1] != n)
+	error(_("determinant of non-square matrix is undefined"));
+    UNPROTECT(1); /* dim */
+    SEXP res;
+    if (n == 0) {
+	int givelog = asLogical(logarithm), sign = 1;
+	double modulus = (givelog) ? 0.0 : 1.0;
+	res = as_det_obj(modulus, givelog, sign);
+    } else {
+	SEXP trf = PROTECT(dgeMatrix_trf_(obj, 0));
+	res = denseLU_determinant(trf, logarithm);
+	UNPROTECT(1); /* trf */
+    }
+    return res;
+}
+
+SEXP dgeMatrix_solve(SEXP a)
+{
+    SEXP dim = PROTECT(GET_SLOT(a, Matrix_DimSym));
+    int *pdim = INTEGER(dim), n = pdim[0];
+    if (pdim[1] != n)
+	error(_("'solve' requires a square matrix"));
+    
+    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
+	dimnames = PROTECT(GET_SLOT(a, Matrix_DimNamesSym)),
+	trf = PROTECT(dgeMatrix_trf_(a, 1)),
+	perm = PROTECT(GET_SLOT(trf, Matrix_permSym)),
+	x;
+    PROTECT_INDEX pid;
+    PROTECT_WITH_INDEX(x = GET_SLOT(trf, Matrix_xSym), &pid);
+    REPROTECT(x = duplicate(x), pid);
+
+    SET_SLOT(val, Matrix_DimSym, dim);
+    set_reversed_DimNames(val, dimnames);
+    SET_SLOT(val, Matrix_xSym, x);
+    
+    if (n > 0) {
+        /* Is matrix _computationally_ singular ? */
+        double *px = REAL(x), norm = get_norm_dge(a, "1"), rcond;
+	int info;
+        F77_CALL(dgecon)("1", pdim, px, pdim, &norm, &rcond,
+                         (double *) R_alloc((size_t) 4 * n, sizeof(double)),
+                         (int *) R_alloc((size_t) n, sizeof(int)),
+			 &info FCONE);
+        if (info)
+            error(_("LAPACK routine '%s' returned with error code %d"),
+		  "dgecon", info);
+        if (rcond < DBL_EPSILON)
+            error(_("LAPACK '%s': matrix is computationally singular, "
+		    "with reciprocal condition number %g"),
+		  "dgecon", rcond);
+	/* Only now try inversion and test if matrix is _exactly_ singular : */
+	double tmp;
+	int *pperm = INTEGER(perm), lwork = -1;
+	F77_CALL(dgetri)(pdim, px, pdim, pperm, &tmp, &lwork, &info);
+	lwork = (int) tmp;
+	F77_CALL(dgetri)(pdim, px, pdim, pperm,
+			 (double *) R_alloc((size_t) lwork, sizeof(double)),
+			 &lwork, &info);
+	if (info)
+	    error(_("LAPACK '%s': matrix is exactly singular"),
+		  "dgetri");
+    }
+    
+    UNPROTECT(6);
+    return val;
+}
+
+SEXP dgeMatrix_matrix_solve(SEXP a, SEXP b)
+{
+    SEXP val = PROTECT(dense_as_general(b, 'd', 2, 0)),
+	adim = PROTECT(GET_SLOT(a, Matrix_DimSym)),
+	bdim = PROTECT(GET_SLOT(val, Matrix_DimSym));
+    int *padim = INTEGER(adim), *pbdim = INTEGER(bdim);
+    
+    if (padim[0] != pbdim[0] || padim[0] < 1 || pbdim[1] < 1)
+	error(_("dimensions of system to be solved are inconsistent"));
+    
+    SEXP trf = PROTECT(dgeMatrix_trf_(a, 1)),
+	perm = PROTECT(GET_SLOT(trf, Matrix_permSym)),
+	x = PROTECT(GET_SLOT(trf, Matrix_xSym)),
+	y = PROTECT(GET_SLOT(val, Matrix_xSym));
+    
+    int *pperm = INTEGER(perm), info;
+    double *px = REAL(x), *py = REAL(y);
+    
+    if (pbdim[0] >= 1 && pbdim[1] >= 1) {
+	F77_CALL(dgetrs)("N", pbdim, pbdim + 1, px, pbdim, pperm, py, pbdim,
+			 &info FCONE);
+	if (info)
+	    error(_("LAPACK '%s': matrix is exactly singular"),
+		  "dgetrs");
+    }
+    
+    UNPROTECT(7);
+    return val;
+}
+
 SEXP dgeMatrix_crossprod(SEXP x, SEXP trans)
 {
 #define DGE_CROSS_1							\
@@ -114,12 +267,12 @@ double* gematrix_real_x(SEXP x, int nn) {
     // else : 'l' or 'n' (for now !!)
     int *xi = INTEGER(GET_SLOT(x, Matrix_xSym));
     double *x_x;
-    C_or_Alloca_TO(x_x, nn, double);
+    Calloc_or_Alloca_TO(x_x, nn, double);
     for(int i=0; i < nn; i++)
 	x_x[i] = (double) xi[i];
 
-    // FIXME: this is not possible either; the *caller* would have to Free(.)
-    if(nn >= SMALL_4_Alloca) Free(x_x);
+    // FIXME: this is not possible either; the *caller* would have to R_Free(.)
+    Free_FROM(x_x, nn);
 #else
     // ideally should be PROTECT()ed ==> make sure R does not run gc() now!
     double *x_x = REAL(coerceVector(GET_SLOT(x, Matrix_xSym), REALSXP));
@@ -137,7 +290,7 @@ SEXP _geMatrix_crossprod(SEXP x, SEXP trans)
 
 SEXP geMatrix_crossprod(SEXP x, SEXP trans)
 {
-    SEXP y = PROTECT(dup_mMatrix_as_geMatrix(x)),
+    SEXP y = PROTECT(dense_as_general(x, '.', 2, 0)),
 	val = _geMatrix_crossprod(y, trans);
     UNPROTECT(1);
     return val;
@@ -202,8 +355,8 @@ SEXP _geMatrix__geMatrix_crossprod(SEXP x, SEXP y, SEXP trans)
 
 SEXP geMatrix_geMatrix_crossprod(SEXP x, SEXP y, SEXP trans)
 {
-    SEXP gx = PROTECT(dup_mMatrix_as_geMatrix(x)),
-	 gy = PROTECT(dup_mMatrix_as_geMatrix(y)),
+    SEXP gx = PROTECT(dense_as_general(x, '.', 2, 0)),
+	gy = PROTECT(dense_as_general(y, '.', 2, 0)),
 	val = _geMatrix__geMatrix_crossprod(gx, gy, trans);
     UNPROTECT(2);
     return val;
@@ -290,7 +443,7 @@ SEXP _geMatrix_matrix_crossprod(SEXP x, SEXP y, SEXP trans) {
 }
 
 SEXP geMatrix_matrix_crossprod(SEXP x, SEXP y, SEXP trans) {
-    SEXP dx = PROTECT(dup_mMatrix_as_geMatrix(x)),
+    SEXP dx = PROTECT(dense_as_general(x, '.', 2, 0)),
 	val = _geMatrix_matrix_crossprod(dx, y, trans);
     UNPROTECT(1);
     return val;
@@ -346,7 +499,7 @@ SEXP dgeMatrix_matrix_mm(SEXP a, SEXP bP, SEXP right)
     UNPROTECT(nprot);							\
     return val
 
-    SEXP b = PROTECT(mMatrix_as_dgeMatrix(bP));
+    SEXP b = PROTECT(dense_as_general(bP, 'd', 2, 0));
     DGE_MAT_MM_1(1);
     DGE_MAT_MM_DO(REAL(GET_SLOT(a, Matrix_xSym)),
                   REAL(GET_SLOT(b, Matrix_xSym)));
@@ -363,14 +516,15 @@ SEXP _geMatrix_matrix_mm(SEXP a, SEXP b, SEXP right) {
 //! %*% -- generalized from dge to *ge():
 SEXP geMatrix_matrix_mm(SEXP a, SEXP b, SEXP right) {
     SEXP
-	da = PROTECT(dup_mMatrix_as_geMatrix(a)),
-	db = PROTECT(dup_mMatrix_as_geMatrix(b)),
+	da = PROTECT(dense_as_general(a, '.', 2, 0)),
+	db = PROTECT(dense_as_general(b, '.', 2, 0)),
 	val = _geMatrix_matrix_mm(da, db, right);
     UNPROTECT(2);
     return val;
 }
 
-//---------------------------------------------------------------------
+/* MJ: no longer needed ... prefer more general unpackedMatrix_diag_[gs]et() */
+#if 0
 
 SEXP dgeMatrix_getDiag(SEXP x)
 {
@@ -409,7 +563,6 @@ SEXP lgeMatrix_getDiag(SEXP x)
 #undef geMatrix_getDiag_1
 #undef geMatrix_getDiag_2
 
-
 SEXP dgeMatrix_setDiag(SEXP x, SEXP d)
 {
 #define geMatrix_setDiag_1					\
@@ -447,6 +600,7 @@ SEXP lgeMatrix_setDiag(SEXP x, SEXP d)
 #undef geMatrix_setDiag_1
 #undef geMatrix_setDiag_2
 
+/* was unused, not replaced: */
 SEXP dgeMatrix_addDiag(SEXP x, SEXP d)
 {
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
@@ -465,145 +619,7 @@ SEXP dgeMatrix_addDiag(SEXP x, SEXP d)
     return ret;
 }
 
-
-
-SEXP dgeMatrix_LU_(SEXP x, Rboolean warn_sing)
-{
-    SEXP val = get_factors(x, "LU");
-    int *dims, npiv, info;
-
-    if (val != R_NilValue) /* nothing to do if it's there in 'factors' slot */
-	return val;
-    dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
-    if (dims[0] < 1 || dims[1] < 1)
-	error(_("Cannot factor a matrix with zero extents"));
-    npiv = (dims[0] < dims[1]) ? dims[0] : dims[1];
-    val = PROTECT(NEW_OBJECT_OF_CLASS("denseLU"));
-    slot_dup(val, x, Matrix_xSym);
-    slot_dup(val, x, Matrix_DimSym);
-    slot_dup(val, x, Matrix_DimNamesSym);
-    F77_CALL(dgetrf)(dims, dims + 1, REAL(GET_SLOT(val, Matrix_xSym)),
-		     dims,
-		     INTEGER(ALLOC_SLOT(val, Matrix_permSym, INTSXP, npiv)),
-		     &info);
-    if (info < 0)
-	error(_("Lapack routine %s returned error code %d"), "dgetrf", info);
-    else if (info > 0 && warn_sing)
-	warning(_("Exact singularity detected during LU decomposition: %s, i=%d."),
-		"U[i,i]=0", info);
-    UNPROTECT(1);
-    return set_factors(x, val, "LU");
-}
-// FIXME: also allow an interface to LAPACK's  dgesvx()  which uses LU fact.
-//        and then optionally does "equilibration" (row and column scaling)
-//  maybe also allow low-level interface to  dgeEQU() ...
-
-SEXP dgeMatrix_LU(SEXP x, SEXP warn_singularity)
-{
-    return dgeMatrix_LU_(x, asLogical(warn_singularity));
-}
-
-SEXP dgeMatrix_determinant(SEXP x, SEXP logarithm)
-{
-    int lg = asLogical(logarithm);
-    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
-	n = dims[0], sign = 1;
-    double modulus = lg ? 0. : 1; /* initialize; = result for n == 0 */
-
-    if (n != dims[1])
-	error(_("Determinant requires a square matrix"));
-    if (n > 0) {
-	SEXP lu = dgeMatrix_LU_(x, /* do not warn about singular LU: */ FALSE);
-	int i, *jpvt = INTEGER(GET_SLOT(lu, Matrix_permSym));
-	double *luvals = REAL(GET_SLOT(lu, Matrix_xSym));
-
-	for (i = 0; i < n; i++) if (jpvt[i] != (i + 1)) sign = -sign;
-	if (lg) {
-	    for (i = 0; i < n; i++) {
-		double dii = luvals[i*(n + 1)]; /* ith diagonal element */
-		modulus += log(dii < 0 ? -dii : dii);
-		if (dii < 0) sign = -sign;
-	    }
-	} else {
-	    for (i = 0; i < n; i++)
-		modulus *= luvals[i*(n + 1)];
-	    if (modulus < 0) {
-		modulus = -modulus;
-		sign = -sign;
-	    }
-	}
-    }
-    return as_det_obj(modulus, lg, sign);
-}
-
-SEXP dgeMatrix_solve(SEXP a)
-{
-    /*  compute the 1-norm of the matrix, which is needed
-	later for the computation of the reciprocal condition number. */
-    double aNorm = get_norm(a, "1");
-
-    /* the LU decomposition : */
-    SEXP val = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
-	lu = dgeMatrix_LU_(a, TRUE);
-    int *dims = INTEGER(GET_SLOT(lu, Matrix_DimSym)),
-	*pivot = INTEGER(GET_SLOT(lu, Matrix_permSym));
-
-    /* prepare variables for the dgetri calls */
-    double *x, tmp;
-    int	info, lwork = -1;
-
-
-    if (dims[0] != dims[1]) error(_("Solve requires a square matrix"));
-    slot_dup(val, lu, Matrix_xSym);
-    x = REAL(GET_SLOT(val, Matrix_xSym));
-    slot_dup(val, lu, Matrix_DimSym);
-
-    if(dims[0]) /* the dimension is not zero */
-    {
-        /* is the matrix is *computationally* singular ? */
-        double rcond;
-        F77_CALL(dgecon)("1", dims, x, dims, &aNorm, &rcond,
-                         (double *) R_alloc(4*dims[0], sizeof(double)),
-                         (int *) R_alloc(dims[0], sizeof(int)), &info FCONE);
-        if (info)
-            error(_("error [%d] from Lapack 'dgecon()'"), info);
-        if(rcond < DOUBLE_EPS)
-            error(_("Lapack dgecon(): system computationally singular, reciprocal condition number = %g"),
-		  rcond);
-
-        /* only now try the inversion and check if the matrix is *exactly* singular: */
-	F77_CALL(dgetri)(dims, x, dims, pivot, &tmp, &lwork, &info);
-	lwork = (int) tmp;
-	F77_CALL(dgetri)(dims, x, dims, pivot,
-			 (double *) R_alloc((size_t) lwork, sizeof(double)),
-			 &lwork, &info);
-	if (info)
-	    error(_("Lapack routine dgetri: system is exactly singular"));
-    }
-    UNPROTECT(1);
-    return val;
-}
-
-SEXP dgeMatrix_matrix_solve(SEXP a, SEXP b)
-{
-    SEXP val = PROTECT(dup_mMatrix_as_dgeMatrix(b)),
-	lu = PROTECT(dgeMatrix_LU_(a, TRUE));
-    int *adims = INTEGER(GET_SLOT(lu, Matrix_DimSym)),
-	*bdims = INTEGER(GET_SLOT(val, Matrix_DimSym));
-    int info, n = bdims[0], nrhs = bdims[1];
-
-    if (adims[0] != n || adims[1] != n)
-	error(_("Dimensions of system to be solved are inconsistent"));
-    if(nrhs >= 1 && n >= 1) {
-	F77_CALL(dgetrs)("N", &n, &nrhs, REAL(GET_SLOT(lu, Matrix_xSym)), &n,
-			 INTEGER(GET_SLOT(lu, Matrix_permSym)),
-			 REAL(GET_SLOT(val, Matrix_xSym)), &n, &info FCONE);
-	if (info)
-	    error(_("Lapack routine dgetrs: system is exactly singular"));
-    }
-    UNPROTECT(2);
-    return val;
-}
+#endif /* MJ */
 
 SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 {
@@ -620,7 +636,7 @@ SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 	int *iwork, n_iw = 8 * mm;
 	if(8 * (double)mm != n_iw) // integer overflow
 	    error(_("dgeMatrix_svd(x,*): dim(x)[j] = %d is too large"), mm);
-	C_or_Alloca_TO(iwork, n_iw, int);
+	Calloc_or_Alloca_TO(iwork, n_iw, int);
 
 	SET_VECTOR_ELT(val, 0, allocVector(REALSXP, mm));
 	SET_VECTOR_ELT(val, 1, allocMatrix(REALSXP, m, mm));
@@ -631,7 +647,7 @@ SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 			 REAL(VECTOR_ELT(val, 2)), &mm,
 			 &tmp, &lwork, iwork, &info FCONE);
 	lwork = (int) tmp;
-	C_or_Alloca_TO(work, lwork, double);
+	Calloc_or_Alloca_TO(work, lwork, double);
 
 	F77_CALL(dgesdd)("S", &m, &n, xx, &m,
 			 REAL(VECTOR_ELT(val, 0)),
@@ -639,8 +655,8 @@ SEXP dgeMatrix_svd(SEXP x, SEXP nnu, SEXP nnv)
 			 REAL(VECTOR_ELT(val, 2)), &mm,
 			 work, &lwork, iwork, &info FCONE);
 
-	if(n_iw  >= SMALL_4_Alloca) Free(iwork);
-	if(lwork >= SMALL_4_Alloca) Free(work);
+	Free_FROM(iwork, n_iw);
+	Free_FROM(work, lwork);
     }
     UNPROTECT(1);
     return val;
@@ -675,13 +691,13 @@ SEXP dgeMatrix_exp(SEXP x)
 
     SEXP val = PROTECT(duplicate(x));
     int i, ilo, ilos, ihi, ihis, j, sqpow;
-    int *pivot = Calloc(n, int);
-    double *dpp = Calloc(nsqr, double), /* denominator power Pade' */
-	*npp = Calloc(nsqr, double), /* numerator power Pade' */
-	*perm = Calloc(n, double),
-	*scale = Calloc(n, double),
+    int *pivot = R_Calloc(n, int);
+    double *dpp = R_Calloc(nsqr, double), /* denominator power Pade' */
+	*npp = R_Calloc(nsqr, double), /* numerator power Pade' */
+	*perm = R_Calloc(n, double),
+	*scale = R_Calloc(n, double),
 	*v = REAL(GET_SLOT(val, Matrix_xSym)),
-	*work = Calloc(nsqr, double), inf_norm, m1_j/*= (-1)^j */, trshift;
+	*work = R_Calloc(nsqr, double), inf_norm, m1_j/*= (-1)^j */, trshift;
     R_CheckStack();
 
     if (n < 1 || Dims[0] != n)
@@ -717,8 +733,8 @@ SEXP dgeMatrix_exp(SEXP x)
     }
 
     /* Pade' approximation. Powers v^8, v^7, ..., v^1 */
-    AZERO(npp, nsqr);
-    AZERO(dpp, nsqr);
+    Matrix_memset(npp, 0, nsqr, sizeof(double));
+    Matrix_memset(dpp, 0, nsqr, sizeof(double));
     m1_j = -1;
     for (j = 7; j >=0; j--) {
 	double mult = padec[j];
@@ -793,7 +809,7 @@ SEXP dgeMatrix_exp(SEXP x)
     }
 
     /* Clean up */
-    Free(work); Free(scale); Free(perm); Free(npp); Free(dpp); Free(pivot);
+    R_Free(work); R_Free(scale); R_Free(perm); R_Free(npp); R_Free(dpp); R_Free(pivot);
     UNPROTECT(1);
     return val;
 }
@@ -834,18 +850,21 @@ SEXP dgeMatrix_Schur(SEXP x, SEXP vectors, SEXP isDGE)
 		    &tmp, &lwork, (int *) NULL, &info FCONE FCONE);
     if (info) error(_("dgeMatrix_Schur: first call to dgees failed"));
     lwork = (int) tmp;
-    C_or_Alloca_TO(work, lwork, double);
+    Calloc_or_Alloca_TO(work, lwork, double);
 
     F77_CALL(dgees)(vecs ? "V" : "N", "N", NULL, dims, REAL(VECTOR_ELT(val, 2)), dims,
 		    &izero, REAL(VECTOR_ELT(val, 0)), REAL(VECTOR_ELT(val, 1)),
 		    REAL(VECTOR_ELT(val, 3)), dims, work, &lwork,
 		    (int *) NULL, &info FCONE FCONE);
-    if(lwork >= SMALL_4_Alloca) Free(work);
+    Free_FROM(work, lwork);
     if (info) error(_("dgeMatrix_Schur: dgees returned code %d"), info);
     UNPROTECT(nprot);
     return val;
 } // dgeMatrix_Schur
 
+/* MJ: no longer needed ... prefer more general R_dense_(col|row)Sums() */
+#if 0
+
 // colSums(), colMeans(), rowSums() and rowMeans() -- called from ../R/colSums.R
 SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
 {
@@ -878,7 +897,7 @@ SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
     } else { /* row(Sums|Means) : */
 	Rboolean do_count = (!keepNA) && doMean;
 	int *cnt = (int*) NULL;
-	if(do_count) { C_or_Alloca_TO(cnt, m, int); }
+	if (do_count) Calloc_or_Alloca_TO(cnt, m, int);
 
 	// (taking care to access x contiguously: vary i inside j)
 	for (i = 0; i < m; i++) {
@@ -904,7 +923,7 @@ SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
 		for (i = 0; i < m; i++)
 		    aa[i] = (cnt[i] > 0) ? aa[i]/cnt[i] : NA_REAL;
 	}
-	if(do_count && m >= SMALL_4_Alloca) Free(cnt);
+	if (do_count) Free_FROM(cnt, m);
     }
 
     SEXP nms = VECTOR_ELT(GET_SLOT(x, Matrix_DimNamesSym), useCols ? 1 : 0);
@@ -913,3 +932,5 @@ SEXP dgeMatrix_colsums(SEXP x, SEXP naRmP, SEXP cols, SEXP mean)
     UNPROTECT(1);
     return ans;
 }
+
+#endif /* MJ */
