@@ -65,6 +65,7 @@ internal_chm_factor(SEXP Ap, int perm, int LDL, int super, double Imult)
     SEXP nms = PROTECT(getAttrib(facs, R_NamesSymbol)); // being very careful..
     CHM_FR L;
     CHM_SP A = AS_CHM_SP__(Ap);
+    double mm[2] = {0, 0}; mm[0] = Imult;
     R_CheckStack();
 
     CHM_store_common();		/* save settings from c */
@@ -75,7 +76,7 @@ internal_chm_factor(SEXP Ap, int perm, int LDL, int super, double Imult)
 		R_CheckStack();
 		/* copy the factor so later it can safely be cholmod_free'd */
 		L = cholmod_copy_factor(L, &c);
-		if (Imult) cholmod_factorize_p(A, &Imult, (int*)NULL, 0, L, &c);
+		if (Imult) cholmod_factorize_p(A, mm, (int*)NULL, 0, L, &c);
 		UNPROTECT(1);
 		return L;
 	    }
@@ -95,7 +96,7 @@ internal_chm_factor(SEXP Ap, int perm, int LDL, int super, double Imult)
 	// *_restore_*() below or in R_cholmod_error() will restore c.<foo>
 	L = cholmod_analyze(A, &c);
     }
-    if (!cholmod_factorize_p(A, &Imult, (int*)NULL, 0 /*fsize*/, L, &c))
+    if (!cholmod_factorize_p(A, mm, (int*)NULL, 0 /*fsize*/, L, &c))
 	// have never seen this, rather R_cholmod_error(status, ..) is called :
 	error(_("Cholesky factorization failed; unusually, please report to Matrix-authors"));
 
@@ -113,7 +114,7 @@ internal_chm_factor(SEXP Ap, int perm, int LDL, int super, double Imult)
 	char fnm[12] = "...Cholesky";// 11 + final \0
 	chm_factor_name(fnm, perm, LDL, super);
 
-	set_factors(Ap, chm_factor_to_SEXP(L, 0), fnm);
+	set_factor(Ap, fnm, chm_factor_to_SEXP(L, 0));
     }
     CHM_restore_common();
     UNPROTECT(1);
@@ -132,8 +133,8 @@ SEXP dsCMatrix_chol(SEXP x, SEXP pivot)
     R = cholmod_transpose(Rt, /*values*/ 1, &c);
     cholmod_free_sparse(&Rt, &c);
     ans = PROTECT(chm_sparse_to_SEXP(R, 1/*do_free*/, 1/*uploT*/, 0/*Rkind*/,
-				     "N"/*diag*/, GET_SLOT(x, Matrix_DimNamesSym)));
-
+				     "N"/*diag*/, R_NilValue));
+    set_symmetrized_DimNames(ans, GET_SLOT(x, Matrix_DimNamesSym), -1);
     if (pivP) {
 	SEXP piv = PROTECT(allocVector(INTSXP, L->n)),
 	     L_n = PROTECT(ScalarInteger((size_t) L->minor));
@@ -231,7 +232,7 @@ SEXP dsCMatrix_matrix_solve(SEXP a, SEXP b, SEXP LDL)
 	return R_NilValue;// == "CHOLMOD factorization failed"
     }
 
-    CHM_DN cx, cb = AS_CHM_DN(PROTECT(mMatrix_as_dgeMatrix(b)));
+    CHM_DN cx, cb = AS_CHM_DN(PROTECT(dense_as_general(b, 'd', 2, 0)));
     R_CheckStack();
     cx = cholmod_solve(CHOLMOD_A, L, cb, &c);
     cholmod_free_factor(&L, &c);
@@ -239,9 +240,9 @@ SEXP dsCMatrix_matrix_solve(SEXP a, SEXP b, SEXP LDL)
     return chm_dense_to_SEXP(cx, 1, 0, /*dimnames = */ R_NilValue, /* transp: */ FALSE);
 }
 
-/* Needed for printing dsCMatrix objects */
-/* FIXME: Create a more general version of this operation: also for lsC, (dsR?),..
-*         e.g. make  compressed_to_dgTMatrix() in ./dgCMatrix.c work for dsC */
+/* MJ: no longer used ... prefer R_sparse_as_general(), CRsparse_as_Tsparse() */
+#if 0
+
 SEXP dsCMatrix_to_dgTMatrix(SEXP x)
 {
     CHM_SP A = AS_CHM_SP__(x);
@@ -255,3 +256,5 @@ SEXP dsCMatrix_to_dgTMatrix(SEXP x)
     return chm_triplet_to_SEXP(At, 1, /*uploT*/ 0, /*Rkind*/ 0, "",
 			       GET_SLOT(x, Matrix_DimNamesSym));
 }
+
+#endif /* MJ */
