@@ -1,42 +1,35 @@
-#include <Rinternals.h>
-#include <R_ext/Lapack.h>
+#include <Rinternals.h> /* SEXP, Rcomplex */
 #include "abIndex.h"
 #include "chm_common.h"
 #include "CHMfactor.h"
 #include "Csparse.h"
-#include "Tsparse.h"
 #include "dense.h"
 #include "dgCMatrix.h"
-#include "dgTMatrix.h"
 #include "dgeMatrix.h"
 #include "dpoMatrix.h"
 #include "dppMatrix.h"
 #include "dsCMatrix.h"
-#include "TMatrix_as.h"
 #include "dspMatrix.h"
 #include "dsyMatrix.h"
 #include "dtCMatrix.h"
-#include "dtTMatrix.h"
 #include "dtrMatrix.h"
 #include "dtpMatrix.h"
 #include "factorizations.h"
-#include "ldense.h"
-#include "lgCMatrix.h"
 #include "sparseQR.h"
 #include "packedMatrix.h"
 #include "unpackedMatrix.h"
 #include "sparse.h"
+#include "subscript.h"
+#include "validity.h"
 #include <R_ext/Rdynload.h>
 
 #include "Syms.h"
-Rcomplex Matrix_zzero, Matrix_zone;
+Rcomplex Matrix_zzero, Matrix_zone, Matrix_zna;
 
 #define CALLDEF(name, n)  {#name, (DL_FUNC) &name, n}
 #define EXTDEF(name, n)   {#name, (DL_FUNC) &name, n}
 
 static R_CallMethodDef CallEntries[] = {
-    CALLDEF(BunchKaufman_validate, 1),
-    CALLDEF(pBunchKaufman_validate, 1),
     CALLDEF(CHMfactor_to_sparse, 1),
     CALLDEF(CHMfactor_solve, 3),
     CALLDEF(CHMfactor_spsolve, 3),
@@ -45,13 +38,12 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(CHMfactor_update, 3),
     CALLDEF(CHMfactor_updown,3),
     CALLDEF(destructive_CHM_update, 3),
-    CALLDEF(Cholesky_validate, 1),
     CALLDEF(Csparse_Csparse_prod, 3),
     CALLDEF(Csparse_Csparse_crossprod, 4),
     CALLDEF(Csparse_MatrixMarket, 2),
     
 /* MJ: no longer needed ... prefer R_sparse_band() */
-/* MJ: however, some reverse dependencies built with Matrix < 1.4-2 need it */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
 #ifdef Matrix_SupportingCachedMethods
     CALLDEF(Csparse_band, 3),
 #endif
@@ -59,8 +51,6 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(Csparse_crossprod, 4),
     CALLDEF(Csparse_dense_crossprod, 3),
     CALLDEF(Csparse_dense_prod, 3),
-    CALLDEF(Csparse_diagN2U, 1),
-    CALLDEF(Csparse_diagU2N, 1),
     CALLDEF(Csparse_drop, 2),
     CALLDEF(Csparse_horzcat, 2),
     CALLDEF(Csparse_sort, 1),
@@ -68,9 +58,8 @@ static R_CallMethodDef CallEntries[] = {
 /* MJ: no longer needed ... prefer CRsparse_to_Tsparse() */
 #if 0
     CALLDEF(compressed_to_TMatrix, 2),
-#endif
-
     CALLDEF(Csparse_to_Tsparse, 2),
+#endif
     
 /* MJ: unused */
 #if 0
@@ -117,27 +106,26 @@ static R_CallMethodDef CallEntries[] = {
 #endif    
 
 /* MJ: no longer needed ... prefer R_sparse_transpose() */
-/* MJ: however, some reverse dependencies built with Matrix < 1.4-2 need it */
+/* MJ: however, some reverse dependencies built with Matrix < 1.5-0 need it */
 #ifdef Matrix_SupportingCachedMethods
     CALLDEF(Csparse_transpose, 2),
 #endif
     
-    CALLDEF(Csparse_validate, 1),
     CALLDEF(Csparse_validate2, 2),
     CALLDEF(Csparse_vertcat, 2),
     CALLDEF(Csparse_dmperm, 3),
-    CALLDEF(pCholesky_validate, 1),
-    CALLDEF(Rsparse_validate, 1),
     CALLDEF(diag_tC, 2),
+
+/* MJ: no longer needed ... prefer denseLU_expand() */
+#if 0
     CALLDEF(LU_expand, 1),
-    CALLDEF(LU_validate, 1),
+#endif
 
 /* MJ: no longer needed ... prefer R_dense_as_sparse() */
 #if 0
     CALLDEF(matrix_to_Csparse, 2),
 #endif
     
-    CALLDEF(MatrixFactorization_validate, 1),
     CALLDEF(Matrix_expand_pointers, 1),
     CALLDEF(R_rbind2_vector, 2),
     CALLDEF(R_all0, 1),
@@ -148,14 +136,14 @@ static R_CallMethodDef CallEntries[] = {
 #if 0
     CALLDEF(R_to_CMatrix, 1),
 #endif
-    
-#ifdef _Matrix_has_SVD_
-    CALLDEF(SVD_validate, 1),
+
+/* MJ: no longer needed ... prefer R_sparse_diag_(U2N|N2U)() */
+#if 0
+    CALLDEF(Csparse_diagU2N, 1),
+    CALLDEF(Csparse_diagN2U, 1),
+    CALLDEF(Tsparse_diagU2N, 1),
 #endif
     
-    CALLDEF(Tsparse_validate, 1),
-    CALLDEF(Tsparse_diagU2N, 1),
-
 /* MJ: no longer needed ... prefer Tsparse_as_CRsparse() */
 #if 0
     CALLDEF(Tsparse_to_Csparse, 2),
@@ -194,10 +182,15 @@ static R_CallMethodDef CallEntries[] = {
 #ifdef Matrix_with_SPQR
     CALLDEF(dgCMatrix_SPQR, 4),
 #endif
+
+/* MJ: no longer needed ... prefer CRsparse_(col|row)Sums() */
+#if 0
     CALLDEF(dgCMatrix_colSums, 5),
     CALLDEF(igCMatrix_colSums, 5),
     CALLDEF(lgCMatrix_colSums, 5),
     CALLDEF(ngCMatrix_colSums, 5),
+#endif
+    
     CALLDEF(dgCMatrix_cholsol, 2),
     /* CALLDEF(dgCMatrix_lusol, 2), */
     CALLDEF(dgCMatrix_matrix_solve, 3),
@@ -215,20 +208,28 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(lgTMatrix_to_matrix, 1),
 #endif
 
-    CALLDEF(dgeMatrix_LU, 2),
-    CALLDEF(dgeMatrix_Schur, 3),
+/* MJ: no longer needed ... prefer R_dense_(col|row)Sums() */
+#if 0
     CALLDEF(dgeMatrix_colsums, 4),
+#endif
+    
+    CALLDEF(dgeMatrix_trf, 2),
+    CALLDEF(dgeMatrix_norm, 2),
+    CALLDEF(dgeMatrix_rcond, 2),
+    CALLDEF(dgeMatrix_determinant, 2),
+    CALLDEF(dgeMatrix_solve, 1),
+    CALLDEF(dgeMatrix_matrix_solve, 2),
     CALLDEF(dgeMatrix_crossprod, 2),
     CALLDEF (geMatrix_crossprod, 2),
-    CALLDEF(dgeMatrix_determinant, 2),
     CALLDEF(dgeMatrix_dgeMatrix_crossprod, 3),
     CALLDEF (geMatrix_geMatrix_crossprod, 3),
+    CALLDEF(dgeMatrix_matrix_crossprod, 3),
+    CALLDEF (geMatrix_matrix_crossprod, 3),
     CALLDEF(dgeMatrix_matrix_mm, 3),
     CALLDEF (geMatrix_matrix_mm, 3),
-    CALLDEF(dgeMatrix_matrix_solve, 2),
-    CALLDEF(dgeMatrix_dtpMatrix_mm, 2),
+    CALLDEF(dgeMatrix_Schur, 3),
     CALLDEF(dgeMatrix_exp, 1),
-
+    
 /* MJ: no longer needed ... prefer unpackedMatrix_diag_[gs]et() */
 #if 0
     CALLDEF(dgeMatrix_getDiag, 1),
@@ -239,22 +240,16 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(dgeMatrix_addDiag, 2),
 #endif
     
-    CALLDEF(dgeMatrix_matrix_crossprod, 3),
-    CALLDEF (geMatrix_matrix_crossprod, 3),
-    CALLDEF(dgeMatrix_norm, 2),
-    CALLDEF(dgeMatrix_rcond, 2),
-    CALLDEF(dgeMatrix_solve, 1),
-    CALLDEF(dpoMatrix_chol, 1),
-    CALLDEF(dpoMatrix_matrix_solve, 2),
-    CALLDEF(dpoMatrix_rcond, 2),
+    CALLDEF(dpoMatrix_trf, 2),
+    CALLDEF(dpoMatrix_rcond, 1),
     CALLDEF(dpoMatrix_solve, 1),
-    CALLDEF(dpoMatrix_validate, 1),
-    CALLDEF(corMatrix_validate, 1),
-    CALLDEF(dppMatrix_chol, 1),
-    CALLDEF(dppMatrix_matrix_solve, 2),
-    CALLDEF(dppMatrix_rcond, 2),
+    CALLDEF(dpoMatrix_matrix_solve, 2),
+
+    CALLDEF(dppMatrix_trf, 2),
+    CALLDEF(dppMatrix_rcond, 1),
     CALLDEF(dppMatrix_solve, 1),
-    CALLDEF(dppMatrix_validate, 1),
+    CALLDEF(dppMatrix_matrix_solve, 2),
+
     CALLDEF(R_chkName_Cholesky, 4),
     CALLDEF(R_chm_factor_name, 3),
     CALLDEF(dsCMatrix_Cholesky, 5),
@@ -279,30 +274,33 @@ static R_CallMethodDef CallEntries[] = {
 #if 0    
     CALLDEF(dsyMatrix_as_dspMatrix, 1),
 #endif
+    
 /* MJ: no longer needed ... prefer R_dense_as_matrix() */
 #if 0
     CALLDEF(dsyMatrix_as_matrix, 2),
 #endif
     
-    CALLDEF(dsyMatrix_matrix_mm, 3),
-    CALLDEF(dsyMatrix_matrix_solve, 2),
+    CALLDEF(dsyMatrix_trf, 2),
     CALLDEF(dsyMatrix_norm, 2),
-    CALLDEF(dsyMatrix_rcond, 2),
+    CALLDEF(dsyMatrix_rcond, 1),
+    CALLDEF(dsyMatrix_determinant, 2),
     CALLDEF(dsyMatrix_solve, 1),
-    CALLDEF(dsyMatrix_trf, 1),
-
+    CALLDEF(dsyMatrix_matrix_solve, 2),
+    CALLDEF(dsyMatrix_matrix_mm, 3),
+    
 /* MJ: no longer needed ... prefer packedMatrix_unpack() */
 #if 0
     CALLDEF(dspMatrix_as_dsyMatrix, 1),
     CALLDEF(dtpMatrix_as_dtrMatrix, 1),
 #endif
     
-    CALLDEF(dspMatrix_matrix_mm, 2),
-    CALLDEF(dspMatrix_matrix_solve, 2),
+    CALLDEF(dspMatrix_trf, 2),
     CALLDEF(dspMatrix_norm, 2),
-    CALLDEF(dspMatrix_rcond, 2),
+    CALLDEF(dspMatrix_rcond, 1),
+    CALLDEF(dspMatrix_determinant, 2),
     CALLDEF(dspMatrix_solve, 1),
-    CALLDEF(dspMatrix_trf, 1),
+    CALLDEF(dspMatrix_matrix_solve, 2),
+    CALLDEF(dspMatrix_matrix_mm, 2),
     
 /* MJ: no longer needed ... prefer packedMatrix_diag_[gs]et() */
 #if 0
@@ -341,40 +339,32 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(ltrMatrix_setDiag, 2),
 #endif
     
-    CALLDEF(dtpMatrix_matrix_mm, 4),
-    CALLDEF(dtpMatrix_matrix_solve, 2),
     CALLDEF(dtpMatrix_norm, 2),
     CALLDEF(dtpMatrix_rcond, 2),
     CALLDEF(dtpMatrix_solve, 1),
+    CALLDEF(dtpMatrix_matrix_solve, 2),
+    CALLDEF(dtpMatrix_matrix_mm, 4),
+    CALLDEF(dgeMatrix_dtpMatrix_mm, 2),
 
 /* MJ: no longer needed ... prefer unpackedMatrix_pack() */
 #if 0
     CALLDEF(dtrMatrix_as_dtpMatrix, 1),
 #endif
+    
 /* MJ: no longer needed ... prefer R_dense_as_matrix() */
 #if 0
     CALLDEF(dtrMatrix_as_matrix, 2),
 #endif
     
-    CALLDEF(dtrMatrix_matrix_mm,    4),
-    CALLDEF(dtrMatrix_dtrMatrix_mm, 4),
-    CALLDEF(dtrMatrix_chol2inv, 1),
-    CALLDEF(dtrMatrix_addDiag, 2),
-    CALLDEF(dtrMatrix_matrix_solve, 2),
     CALLDEF(dtrMatrix_norm, 2),
     CALLDEF(dtrMatrix_rcond, 2),
     CALLDEF(dtrMatrix_solve, 1),
+    CALLDEF(dtrMatrix_matrix_solve, 2),
+    CALLDEF(dtrMatrix_dtrMatrix_mm, 4),
+    CALLDEF(dtrMatrix_matrix_mm, 4),
+    CALLDEF(dtrMatrix_chol2inv, 1),
+    CALLDEF(dtrMatrix_addDiag, 2),
     
-    /* for dgC* _and_ lgC* : */
-    CALLDEF(xCMatrix_validate, 1),
-    CALLDEF(xRMatrix_validate, 1),
-    CALLDEF(xTMatrix_validate, 1),
-    CALLDEF(tCMatrix_validate, 1),
-    CALLDEF(tRMatrix_validate, 1),
-    CALLDEF(tTMatrix_validate, 1),
-
-    CALLDEF(lapack_qr, 2),
-
 /* MJ: no longer needed ... prefer R_sparse_as_matrix() */
 #if 0
     CALLDEF(lgC_to_matrix, 1),
@@ -395,32 +385,80 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(lsyMatrix_as_lgeMatrix, 2),
 #endif
 
+    CALLDEF(lapack_qr, 2),
     CALLDEF(lsq_dense_Chol, 2),
     CALLDEF(lsq_dense_QR, 2),
-    CALLDEF(sparseQR_validate, 1),
     CALLDEF(sparseQR_qty, 4),
     CALLDEF(sparseQR_coef, 2),
     CALLDEF(sparseQR_resid_fitted, 3),
 
     CALLDEF(Matrix_validate, 1),
+    CALLDEF(MatrixFactorization_validate, 1),
     CALLDEF(compMatrix_validate, 1),
-    CALLDEF(triangularMatrix_validate, 1),
-    CALLDEF(symmetricMatrix_validate, 1),
-    CALLDEF(diagonalMatrix_validate, 1),
-    CALLDEF(unpackedMatrix_validate, 1),
-    CALLDEF(packedMatrix_validate, 1),
     CALLDEF(dMatrix_validate, 1),
     CALLDEF(lMatrix_validate, 1),
     CALLDEF(ndenseMatrix_validate, 1),
     CALLDEF(iMatrix_validate, 1),
     CALLDEF(zMatrix_validate, 1),
+
+    CALLDEF(symmetricMatrix_validate, 1),
+    CALLDEF(triangularMatrix_validate, 1),
+    CALLDEF(diagonalMatrix_validate, 1),
     CALLDEF(indMatrix_validate, 1),
     CALLDEF(pMatrix_validate, 1),
+    CALLDEF(CsparseMatrix_validate, 1),
+    CALLDEF(RsparseMatrix_validate, 1),
+    CALLDEF(TsparseMatrix_validate, 1),
+
+    CALLDEF(sCMatrix_validate, 1),
+    CALLDEF(tCMatrix_validate, 1),
+    CALLDEF(sRMatrix_validate, 1),
+    CALLDEF(tRMatrix_validate, 1),
+    CALLDEF(sTMatrix_validate, 1),
+    CALLDEF(tTMatrix_validate, 1),
+    
+    CALLDEF(xgCMatrix_validate, 1),
+    CALLDEF(xsCMatrix_validate, 1),
+    CALLDEF(xtCMatrix_validate, 1),
+    CALLDEF(xgRMatrix_validate, 1),
+    CALLDEF(xsRMatrix_validate, 1),
+    CALLDEF(xtRMatrix_validate, 1),
+    CALLDEF(xgTMatrix_validate, 1),
+    CALLDEF(xsTMatrix_validate, 1),
+    CALLDEF(xtTMatrix_validate, 1),
+    
+    CALLDEF(unpackedMatrix_validate, 1),
+    CALLDEF(packedMatrix_validate, 1),
+    CALLDEF(dpoMatrix_validate, 1),
+    CALLDEF(dppMatrix_validate, 1),
+    CALLDEF(corMatrix_validate, 1),
+    CALLDEF(Cholesky_validate, 1),
+    CALLDEF(pCholesky_validate, 1),
+    CALLDEF(BunchKaufman_validate, 1),
+    CALLDEF(pBunchKaufman_validate, 1),
+    CALLDEF(Schur_validate, 1),
+    CALLDEF(denseLU_validate, 1),
+    CALLDEF(sparseLU_validate, 1),
+    CALLDEF(sparseQR_validate, 1),
+    CALLDEF(CHMfactor_validate, 1),
+    CALLDEF(CHMsimpl_validate, 1),
+    CALLDEF(CHMsuper_validate, 1),
+
+/* MJ: some reverse dependencies built with Matrix < 1.5-0 need these */
+#ifdef Matrix_SupportingCachedMethods
+    { "Csparse_validate", (DL_FUNC) &CsparseMatrix_validate, 1},
+    { "Rsparse_validate", (DL_FUNC) &RsparseMatrix_validate, 1},
+    { "Tsparse_validate", (DL_FUNC) &TsparseMatrix_validate, 1},
+    {"xCMatrix_validate", (DL_FUNC)     &xgCMatrix_validate, 1},
+    {"xRMatrix_validate", (DL_FUNC)     &xgRMatrix_validate, 1},
+    {"xTMatrix_validate", (DL_FUNC)     &xgTMatrix_validate, 1},
+    {      "LU_validate", (DL_FUNC)       &denseLU_validate, 1},
+#endif
 
     CALLDEF(R_Dim_validate, 1),
     CALLDEF(R_DimNames_validate, 2),
 
-/* MJ: some reverse dependencies built with Matrix < 1.4-2 need these */
+/* MJ: some reverse dependencies built with Matrix < 1.5-0 need these */
 #ifdef Matrix_SupportingCachedMethods
     {     "Dim_validate", (DL_FUNC)      &R_Dim_validate_old, 2},
     {"dimNames_validate", (DL_FUNC) &R_DimNames_validate_old, 1},
@@ -430,6 +468,7 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(R_DimNames_is_symmetric, 1),
     CALLDEF(R_symmDN, 1),
     CALLDEF(R_revDN, 1),
+    CALLDEF(R_Matrix_nonvirtual, 2),
     CALLDEF(R_Matrix_kind, 2),
     CALLDEF(R_Matrix_shape, 1),
     CALLDEF(R_Matrix_repr, 1),
@@ -437,6 +476,10 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(R_index_diagonal, 3),
     CALLDEF(R_nnz, 3),
 
+    CALLDEF(R_subscript_1ary, 2),
+    CALLDEF(R_subscript_1ary_mat, 2),
+    CALLDEF(R_subscript_2ary, 3),
+
     CALLDEF(R_sparse_as_dense, 2),
     CALLDEF(R_sparse_as_matrix, 1),
     CALLDEF(R_sparse_as_vector, 1),
@@ -450,13 +493,13 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(R_sparse_drop0, 1),
     CALLDEF(R_sparse_band, 3),
     CALLDEF(R_sparse_diag_get, 2),
+    CALLDEF(R_sparse_diag_set, 2),
+    CALLDEF(R_sparse_diag_U2N, 1),
+    CALLDEF(R_sparse_diag_N2U, 1),
     CALLDEF(R_sparse_transpose, 1),
     CALLDEF(R_sparse_force_symmetric, 2),
-
-    CALLDEF(CRsparse_symmpart, 1),
-    CALLDEF(Tsparse_symmpart, 1),
-    CALLDEF(CRsparse_skewpart, 1),
-    CALLDEF(Tsparse_skewpart, 1),
+    CALLDEF(R_sparse_symmpart, 1),
+    CALLDEF(R_sparse_skewpart, 1),
 
     CALLDEF(CRsparse_as_Tsparse, 1),
     CALLDEF(Tsparse_as_CRsparse, 2),
@@ -470,23 +513,28 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(Tsparse_is_triangular, 2),
     CALLDEF(Csparse_is_symmetric, 2),
     CALLDEF(Rsparse_is_symmetric, 2),
+    CALLDEF(CRsparse_colSums, 4),
+    CALLDEF(CRsparse_rowSums, 4),
     
     CALLDEF(R_matrix_as_dense, 4),
     CALLDEF(R_dense_as_general, 2),
     CALLDEF(R_dense_as_sparse, 4),
     CALLDEF(R_dense_as_kind, 2),
-    CALLDEF(R_dense_as_matrix, 2),
+    CALLDEF(R_dense_as_matrix, 1),
     CALLDEF(R_geMatrix_as_matrix, 2),
-    CALLDEF(R_dense_as_vector, 2),
+    CALLDEF(R_dense_as_vector, 1),
     CALLDEF(R_geMatrix_as_vector, 2),
     CALLDEF(R_dense_band, 3),
+    CALLDEF(R_dense_colSums, 3),
+    CALLDEF(R_dense_rowSums, 3),
     
     CALLDEF(matrix_is_symmetric, 2),
     CALLDEF(matrix_is_triangular, 2),
     CALLDEF(matrix_is_diagonal, 1),
     CALLDEF(matrix_symmpart, 1),
     CALLDEF(matrix_skewpart, 1),
-
+    CALLDEF(matrix_trf, 3),
+    
     CALLDEF(unpackedMatrix_pack, 4),
     CALLDEF(unpackedMatrix_force_symmetric, 2),
     CALLDEF(unpackedMatrix_is_symmetric, 2),
@@ -512,18 +560,17 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(packedMatrix_sub1_mat, 2),
     CALLDEF(packedMatrix_sub2, 4),
 
-/* still simple placeholders, but already used in ../R/AllClass.R : */
-    CALLDEF(CHMfactor_validate, 1),
-    CALLDEF(CHMsimpl_validate, 1),
-    CALLDEF(CHMsuper_validate, 1),
-
+    CALLDEF(denseLU_expand, 1),
+    CALLDEF(BunchKaufman_expand, 1),
+    CALLDEF(denseLU_determinant, 2),
+    CALLDEF(BunchKaufman_determinant, 2),
+    
     CALLDEF(CHM_set_common_env, 1),
 
     CALLDEF(inv_permutation, 3),
     CALLDEF(m_encodeInd,  4),
     CALLDEF(m_encodeInd2, 5),
 
-    CALLDEF(matrix_trf, 2),
     CALLDEF(Matrix_rle_i, 2),
     CALLDEF(Matrix_rle_d, 2),
 
@@ -552,8 +599,16 @@ R_init_Matrix(DllInfo *dll)
 
 #define RREGDEF(name)  R_RegisterCCallable("Matrix", #name, (DL_FUNC) name)
 
+#if 0
     RREGDEF(Csparse_diagU2N);
-
+    RREGDEF(dpoMatrix_chol);
+#else
+    R_RegisterCCallable(
+	"Matrix", "Csparse_diagU2N", (DL_FUNC) R_sparse_diag_U2N);
+    R_RegisterCCallable(
+	"Matrix", "dpoMatrix_chol", (DL_FUNC) dpoMatrix_trf);
+#endif
+    
     RREGDEF(as_cholmod_dense);
     RREGDEF(as_cholmod_factor);
     RREGDEF(as_cholmod_factor3);
@@ -605,28 +660,31 @@ R_init_Matrix(DllInfo *dll)
     RREGDEF(cholmod_vertcat);
     RREGDEF(cholmod_updown);
 
-    RREGDEF(dpoMatrix_chol);
     RREGDEF(numeric_as_chm_dense);
 
     R_cholmod_start(&c);
 //    R_cholmod_start(&cl); << TODO; needs more work in ./chm_common.c etc
 
     Matrix_DimNamesSym = install("Dimnames");
-    Matrix_DimSym = install("Dim");
-    Matrix_LSym = install("L");
-    Matrix_RSym = install("R");
-    Matrix_USym = install("U");
-    Matrix_VSym = install("V");
-    Matrix_betaSym = install("beta");
-    Matrix_diagSym = install("diag");
-    Matrix_factorSym = install("factors");
-    Matrix_iSym = install("i");
-    Matrix_jSym = install("j");
-    Matrix_lengthSym = install("length");
-    Matrix_pSym = install("p");
-    Matrix_permSym = install("perm");
-    Matrix_uploSym = install("uplo");
-    Matrix_xSym = install("x");
+    Matrix_DimSym      = install("Dim");
+    Matrix_LSym        = install("L");
+    Matrix_QSym        = install("Q");
+    Matrix_RSym        = install("R");
+    Matrix_TSym        = install("T");
+    Matrix_USym        = install("U");
+    Matrix_VSym        = install("V");
+    Matrix_betaSym     = install("beta");
+    Matrix_diagSym     = install("diag");
+    Matrix_factorSym   = install("factors");
+    Matrix_iSym        = install("i");
+    Matrix_jSym        = install("j");
+    Matrix_lengthSym   = install("length");
+    Matrix_pSym        = install("p");
+    Matrix_permSym     = install("perm");
+    Matrix_qSym        = install("q");
+    Matrix_sdSym       = install("sd");
+    Matrix_uploSym     = install("uplo");
+    Matrix_xSym        = install("x");
     
     Matrix_NS = R_FindNamespace(mkString("Matrix"));
     if(Matrix_NS == R_UnboundValue)
@@ -642,8 +700,8 @@ R_init_Matrix(DllInfo *dll)
 #endif
 	error(_("Matrix namespace not determined correctly"));
 
-    Matrix_zzero.r = 0.0; Matrix_zone.r = 1.0;
-    Matrix_zzero.i = 0.0; Matrix_zone.i = 0.0;
+    Matrix_zzero.r = 0.0; Matrix_zone.r = 1.0; Matrix_zna.r = NA_REAL;
+    Matrix_zzero.i = 0.0; Matrix_zone.i = 0.0; Matrix_zna.i = NA_REAL;
 }
 
 void R_unload_Matrix(DllInfo *dll)
